
const { createApp } = Vue;
let app = createApp({
	data() {
		return {
			logged: false,
			secret: "",
			stops: {},
			devices: []
		}
	},
	methods: {
		async update() {
			let devices = await api("/admin/devices", {secret: this.secret});
			if(devices.error) {
				alert("Neplatné heslo!");
				return;
			}
			let stops = await api("/stops");
			this.devices = devices.devices;
			this.stops = stops.stops;
			this.logged = true;
		},
		async device_clear(index) {
			await api("/admin/devices/clear", {index, secret: this.secret});
		},
		async device_resend(index) {
			await api("/admin/devices/resend", {index, secret: this.secret});
		},
		async device_toggle(index, state) {
			this.devices[index].enabled = state;
			await api("/admin/devices/toggle", {index, state, secret: this.secret});
		},
		async device_change_stop(index) {
			let stop_id = prompt("Zadejte ID zastávky:");
			if(!stop_id || stop_id.trim() == "") return;
			this.devices[index].stop_id = stop_id;
			await api("/admin/devices/change_stop", {index, stop_id, secret: this.secret});
		},
		async device_change_filter(index) {
			let filter = prompt("Zadejte jméno filtru:");
			if(!filter || filter.trim() == "") return;
			this.devices[index].filter = filter;
			await api("/admin/devices/change_filter", {index, filter, secret: this.secret});
		}
	}
}).mount('#app');
async function api(url, data) {
	return new Promise(function(resolve, reject) {
		var xhr = new XMLHttpRequest();
		xhr.onreadystatechange = function() {
			if(this.readyState !== 4) return;
			if(!this.responseText) reject(this);
			try {
				resolve(JSON.parse(this.responseText));
			} catch(e) {
				resolve(null);
			}
		};
		xhr.open(data ? "POST" : "GET", url, true);
		xhr.setRequestHeader("Content-type", "application/x-www-form-urlencoded");
		var str = [];
		for (var key in data) {
			if (data.hasOwnProperty(key)) {
				str.push(encodeURIComponent(key) + "=" + encodeURIComponent(data[key]))
			}
		}
		xhr.send(str.join("&"));
	});
}